//+------------------------------------------------------------------+
//|                                                        renko.mq5 |
//|                        Copyright 2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property version   "1.00"
#include "Panel.mqh"

#property description "Beta version"
#property description ""
#property description "Use arrows, PageUp/PageDown and Home/End for navigation"

#define DATE_BAR_HEIGHT    50
#define FONT_SIZE          12
#define BGCOLOR_PANEL      0xDDDDED
#define BORDER_COLOR_PANEL 0x000000
#define BGCOLOR_GRAPH_1    0xDDDDDD
#define BGCOLOR_GRAPH_2    0xEEEEEE
#define PRICE_COLOR        0x222222

input int InpBoxSize=30; // Box size
input int InpRevSize= 2; // Revers value
input string InpBoxSizes="5,10,15,20,30,40,50,100,150,300";  // Box values
input string InpRevSizes="2,3,4,5"; // Reverse values

int    ExtScaleArray[]={-42,-55,-88,-102,-110,-120,-140,-160,-200,-250,-300,-500};
string ExtBoxValues[];
string ExtRevValues[];
string ExtBuildInBoxValues[]={"5","10","15","20","30","40","50","100","150","300"};
string ExtBuildInRevValues[]={"2","3","4","5","6","7","8","9","10"};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#define DIVR(val,div,up) int(up?MathCeil((val)/double(div)):MathFloor((val)/double(div)))
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct SXOBar
  {
   int               dtb;
   int               dte;
   int               dir;
   int               beg;
   int               end;
   long              vol;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct STrend
  {
   int               nbeg;
   int               vbeg;
   int               nend;
   int               vend;
   int               dir;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct SDateInfo
  {
   datetime          time;
   int               level;
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CXOGraph : public CWindow
  {
   CToolBar          m_tool_bar;

   string            m_symbol;
   ENUM_TIMEFRAMES   m_period;
   datetime          m_time_last;
   datetime          m_time_start;
   int               m_box;
   int               m_reverse;
   int               m_global_min;
   int               m_global_max;
   int               m_rev_min;
   int               m_rev_max;
   int               m_cont_min;
   int               m_cont_max;
   int               m_state;
   int               m_trend_v;
   int               m_font_size;
   int               m_scale;
   int               m_calc_mode;
   int               m_digits;
   int               m_price_width;

   int               m_cell_width;
   int               m_cell_height;
   //int               m_rect.cx;
   //int               m_rect.cy;
   uint              m_graph_p[];
   uint              m_graph_d[];

   SXOBar            m_data[];
   STrend            m_trend[];
   SDateInfo         m_dates[];

   int               m_v_offset;
   int               m_mouse_x;
   int               m_mouse_y;
   int               m_offset_x;
   int               m_offset_y;
   int               m_mouse_key;
   int               m_info_x;
   int               m_info_y;
   MqlTick           m_tick;

   double            m_minval;
   double            m_maxval;

   bool              m_mouse_ctrl;

public:
                     CXOGraph();
                    ~CXOGraph();
   //---
   bool              Initialize();
   void              ChartChange();
   bool              OnMouseMove(int x,int y,uint keys);
   void              NewTick();
   void              Redraw();
   void              KeyDown(int key_code,uint key_state);
   void              SetScale(int x);
   void              UpdatePeriod(string symbol,ENUM_TIMEFRAMES period);
   void              UpdateParameters(int box,int reverse,datetime from,int calc_mode);

private:
   void              Resize();
   void              DrawGrid(int v_shift);
   void              DrawTrendLines(int v_shift);
   void              DrawPrices(int v_shift);
   void              DrawBars(int v_shift);
   void              DrawDates(int v_shift);
   void              Line(int x0,int y0,int x1,int y1,uint clr);
   int               CalcVShift();
   bool              OnNotify(int code,CFrame *sender);
   void              CalcHL();
   void              CalcClose();
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CXOGraph ExtXO;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   StringSplit(InpBoxSizes,',',ExtBoxValues);

   int count=0;   
   for(int i=0;i<ArraySize(ExtBoxValues);i++)
      if((int)ExtBoxValues[i]!=0)
         ExtBoxValues[count++]=(string)(int)ExtBoxValues[i];
   if(count==0) ArrayCopy(ExtBoxValues,ExtBuildInBoxValues);
   else         ArrayResize(ExtBoxValues,count);
      
   count=0;
   StringSplit(InpRevSizes,',',ExtRevValues);
   for(int i=0;i<ArraySize(ExtRevValues);i++)
      if((int)ExtRevValues[i]!=0)
         ExtRevValues[count++]=(string)(int)ExtRevValues[i];
   if(count==0) ArrayCopy(ExtRevValues,ExtBuildInRevValues);
   else         ArrayResize(ExtRevValues,count);
  
   ExtXO.UpdatePeriod(Symbol(),Period());
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---

  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   ExtXO.NewTick();
   ExtXO.Redraw();
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
   switch(id)
     {
      case CHARTEVENT_CHART_CHANGE:
         ExtXO.ChartChange();
         break;
      case CHARTEVENT_MOUSE_MOVE:
         ExtXO.EventMouse((int)lparam,(int)dparam,(uint)sparam);
         break;
      case CHARTEVENT_OBJECT_CLICK:
      case CHARTEVENT_CLICK:
         ExtXO.EventMouse((int)lparam,(int)dparam,0);
         break;
      case CHARTEVENT_KEYDOWN:
         ExtXO.KeyDown((int)lparam,(uint)sparam);
         break;
      default:
         //Print(EnumToString((ENUM_CHART_EVENT)id));
         break;
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CXOGraph::CXOGraph(void)
   :m_offset_x(0),m_mouse_key(0),m_info_x(-1),m_info_y(-1),m_symbol(Symbol())
   ,m_time_last(-1),m_time_start(0),m_period(PERIOD_M1),m_offset_y(0)
   ,m_calc_mode(1),m_price_width(30)
  {
   Initialize();
   SetScale(3);
   Resize();

   MqlDateTime dt;
   TimeLocal(dt);
   dt.day      =1;
   dt.hour     =0;
   dt.min      =0;
   dt.sec      =0;
   m_time_start=StructToTime(dt)-PeriodSeconds(PERIOD_MN1)*60;
   m_box       =InpBoxSize;
   m_reverse   =InpRevSize;
   m_state     =0;
   m_global_max=INT_MIN;
   m_global_min=INT_MAX;
   m_time_last =m_time_start;
   ZeroMemory(m_tick);  
//---
   string name="XO_"+(string)rand();
//---
   m_obj_name=name;
   
   Redraw();
  
   ObjectCreate(0,name,OBJ_BITMAP_LABEL,0,0,0);
   ObjectSetInteger(0,name,OBJPROP_XDISTANCE,0);
   ObjectSetInteger(0,name,OBJPROP_YDISTANCE,0);
   ObjectSetString(0,name,OBJPROP_BMPFILE,"::"+name);
   name=m_obj_name+"_p";
   ObjectCreate(0,name,OBJ_BITMAP_LABEL,0,0,0);
   ObjectSetInteger(0,name,OBJPROP_XDISTANCE,0);
   ObjectSetInteger(0,name,OBJPROP_YDISTANCE,0);
   ObjectSetString(0,name,OBJPROP_BMPFILE,"::"+name);
   name=m_obj_name+"_d";
   ObjectCreate(0,name,OBJ_BITMAP_LABEL,0,0,0);
   ObjectSetInteger(0,name,OBJPROP_XDISTANCE,0);
   ObjectSetInteger(0,name,OBJPROP_YDISTANCE,0);
   ObjectSetString(0,name,OBJPROP_BMPFILE,"::"+name);

   SRect rc={10,5,490,28};   
   m_tool_bar.Create(GetPointer(this),1000,rc,0);
   m_tool_bar.SetBoxValue(m_box);
   m_tool_bar.SetRevValue(m_reverse);
   m_tool_bar.SetModeValue(m_calc_mode);
   m_tool_bar.SetDateValue(m_time_start);
   m_tool_bar.Redraw();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CXOGraph::~CXOGraph(void)
  {
   ObjectDelete(0,m_obj_name);
   ObjectDelete(0,m_obj_name+"_p");
   ObjectDelete(0,m_obj_name+"_d");

   ResourceFree("::"+m_obj_name);
   ResourceFree("::"+m_obj_name+"_p");
   ResourceFree("::"+m_obj_name+"_d");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CXOGraph::Initialize()
  {
   ObjectsDeleteAll(0,-1,OBJ_EVENT);
//---
   ChartSetInteger(0,CHART_SHOW_VOLUMES,false);
   ChartSetInteger(0,CHART_SHOW_OHLC,false);
   ChartSetInteger(0,CHART_SHOW_GRID,false);
   ChartSetInteger(0,CHART_SHOW_VOLUMES,false);
   ChartSetInteger(0,CHART_SHOW_BID_LINE,false);
   ChartSetInteger(0,CHART_SHOW_ASK_LINE,false);
   ChartSetInteger(0,CHART_SHOW_LAST_LINE,false);
   ChartSetInteger(0,CHART_MODE,CHART_LINE);
   ChartSetInteger(0,CHART_COLOR_CHART_LINE,clrNONE);
/**/

   ChartSetInteger(0,CHART_SHOW_PRICE_SCALE,false);
   ChartSetInteger(0,CHART_SHOW_DATE_SCALE,false);
   ChartSetInteger(0,CHART_EVENT_MOUSE_MOVE,true);   
//---
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::UpdatePeriod(string symbol,ENUM_TIMEFRAMES period)
  {
   m_symbol    =symbol;
   m_period    =period;
   m_state     =0;
   m_global_max=INT_MIN;
   m_global_min=INT_MAX;
   m_time_last =m_time_start;
   
   m_digits    =SymbolInfoInteger(m_symbol,SYMBOL_DIGITS)<=3?2:4;   

   NewTick();
   ExtXO.Redraw();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::UpdateParameters(int box,int reverse,datetime from,int calc_mode)
  {
   m_time_start=from;
   m_box       =box;
   m_reverse   =reverse;
   m_state     =0;
   m_global_max=INT_MIN;
   m_global_min=INT_MAX;
   m_time_last =from;
   m_calc_mode =calc_mode;

   NewTick();
   ExtXO.Redraw();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::ChartChange(void)
  {
   Resize();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::Resize(void)
  {
   int cx=(int)ChartGetInteger(0,CHART_WIDTH_IN_PIXELS);
   int cy=(int)ChartGetInteger(0,CHART_HEIGHT_IN_PIXELS);
//---
   if(cx==m_rect.cx && cy==m_rect.cy)
      return;
//---
   if(cx==0 || cy==0)
      return;
//---
   m_rect.cx=cx-m_price_width;
   m_rect.cy=cy-DATE_BAR_HEIGHT;

   if(m_rect.cx<0)
      m_rect.cx=0;
//---
   if(ArrayResize(m_surface,(cx+128)*(cy+128),1024*1024)<0 || 
      ArrayResize(m_graph_p,m_price_width*cy,m_price_width*1024)<0 || 
      ArrayResize(m_graph_d,m_rect.cx*DATE_BAR_HEIGHT,DATE_BAR_HEIGHT*1024)<0)
      return;
//---
   ObjectSetInteger(0,m_obj_name+"_p",OBJPROP_XDISTANCE,m_rect.cx);
   ObjectSetInteger(0,m_obj_name+"_d",OBJPROP_YDISTANCE,m_rect.cy);
//---
   Redraw();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::Redraw(void)
  {
   m_v_offset=CalcVShift();
//---  
   DrawGrid(m_v_offset);
//---   
   DrawDates(m_v_offset);
//---  
   DrawBars(m_v_offset);
//---   
   DrawPrices(m_v_offset);
//---   
   //DrawTrendLines(v_shift);
//---
   ResourceCreate("::"+m_obj_name,m_surface,m_rect.cx,m_rect.cy,0,0,m_rect.cx,COLOR_FORMAT_XRGB_NOALPHA);
   ResourceCreate("::"+m_obj_name+"_p",m_graph_p,m_price_width,m_rect.cy+DATE_BAR_HEIGHT,0,0,m_price_width,COLOR_FORMAT_XRGB_NOALPHA);
   ResourceCreate("::"+m_obj_name+"_d",m_graph_d,m_rect.cx,DATE_BAR_HEIGHT,0,0,m_rect.cx,COLOR_FORMAT_XRGB_NOALPHA);
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::DrawGrid(int v_shift)
  {
   if(m_rect.cy<=DATE_BAR_HEIGHT || m_rect.cx==0) return;
//---      Y
   int dy=v_shift;
   while(dy<0) dy+=m_cell_height*400;
//--- 
   for(int y=0;y<m_rect.cy;y++)
      for(int x=0;x<m_rect.cx;x++)
         m_surface[x+y*m_rect.cx]=bool((((y+dy)/m_cell_height)&1)^(((x+m_offset_x)/m_cell_width)&1))?BGCOLOR_GRAPH_1:BGCOLOR_GRAPH_2;
//---
   if(ArraySize(m_graph_p)>=m_price_width*(m_rect.cy+DATE_BAR_HEIGHT))
      ArrayFill(m_graph_p,0,m_price_width*(m_rect.cy+DATE_BAR_HEIGHT),BGCOLOR_PANEL);
   if(ArraySize(m_graph_d)>=DATE_BAR_HEIGHT*m_rect.cx)
      ArrayFill(m_graph_d,0,DATE_BAR_HEIGHT*m_rect.cx,BGCOLOR_PANEL);
//---
   for(int i=0;i<m_rect.cy;i++)
      m_graph_p[i*m_price_width]=BORDER_COLOR_PANEL;
//---
   if(ArraySize(m_graph_d)>m_rect.cx)
      ArrayFill(m_graph_d,0,m_rect.cx,BORDER_COLOR_PANEL);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CXOGraph::OnMouseMove(int x,int y,uint keys)
  {
//---        
   if((keys&1)==1)
     {
      //---       
      //---     
      if(m_mouse_key==0)
        {
         //---  CTRL?
         m_mouse_ctrl=bool((keys&8)==8);
         m_mouse_key =1;
         m_mouse_x   =x;
         m_mouse_y   =y;
         return(true);
        }
      //---    
      int offset_x=m_offset_x-(x-m_mouse_x);
      int offset_y=m_offset_y-(y-m_mouse_y);
      //---    
      m_mouse_x=x;
      m_mouse_y=y;
      //---
      if(m_mouse_ctrl)
         return(true);
      //---      
      if(offset_x>((ArraySize(m_data)+1)*m_cell_width-m_rect.cx))
         offset_x=(ArraySize(m_data)+1)*m_cell_width-m_rect.cx;
      //---      
      if(offset_x<0)
         offset_x=0;
      //---
      if(offset_y>(m_global_max-m_global_min)*m_cell_height-m_rect.cy)
         offset_y=(m_global_max-m_global_min)*m_cell_height-m_rect.cy;
      //---
      if(offset_y<0)
         offset_y=0;
      //---
      if(offset_y!=m_offset_y || offset_x!=m_offset_x)
        {
         m_offset_x=offset_x;
         m_offset_y=offset_y;
         Redraw();
        }
     }
   else
     {
      //---         
      if(m_mouse_key==1)
        {
         m_mouse_key=0;
         return(true);
        }
      //---      
      x+=m_offset_x;
      y+=m_offset_y;
      //---     
      x/=m_cell_width;
      //---      
      if(x>=ArraySize(m_data))
         x=-1;
      //---     -   
      if(x<0)
        {
         ObjectSetString(0,m_obj_name,OBJPROP_TOOLTIP,NULL);
         ChartRedraw();
        }
      else
        {
         y=m_global_max-(y+m_v_offset)/m_cell_height;        
         //---      
         if(m_info_x!=x || m_info_y!=y)
           {
            //---       
            m_info_x=x;
            m_info_y=y;
            //---   ?
            if(y<=m_data[x].beg || y>m_data[x].end+1)
              {
               ObjectSetString(0,m_obj_name,OBJPROP_TOOLTIP,NULL);
              }
            else
              {
               //---   
               string info=TimeToString(m_dates[m_data[x].dtb].time)+" - "+TimeToString(m_dates[m_data[x].dte].time);
               
               info+="\n\n"+(string)y+": "+DoubleToString((y-1)*m_box/MathPow(10,m_digits),m_digits)+"-"+DoubleToString(y*m_box/MathPow(10,m_digits),m_digits);
               /*
               for(int l=-1,i=m_data[x].dtb;i<=m_data[x].dte;i++)
                 {
                  if(m_dates[i].level!=l)
                    {
                     info+="\n"+string(m_dates[i].level)+"-"+TimeToString(m_dates[i].time);
                     l=m_dates[i].level;
                    }
                 }
               */   
               //Print(y);
   
               ObjectSetString(0,m_obj_name,OBJPROP_TOOLTIP,info);               
              }
            //---
            ChartRedraw();
           }
        }
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::NewTick(void)
  {
   MqlTick tick;
   
   if(SymbolInfoTick(m_symbol,tick))
      m_tick=tick;
  
//---
   if(m_scale<2)
      TextSetFont("Arial",ExtScaleArray[m_scale]);
   else
      TextSetFont("Arial",FONT_SIZE);
//---
   int w,h;
   TextGetSize(DoubleToString(tick.bid,m_digits)+"0",w,h);
   
   if(m_price_width!=w)
     {
      m_rect.cx   +=m_price_width-w;
      m_price_width=w;
      
      ArrayResize(m_surface,(m_rect.cx+128)*(m_rect.cy+128),1024*1024);
      ArrayResize(m_graph_p,m_price_width*m_rect.cy,m_price_width*1024);
      ArrayResize(m_graph_d,m_rect.cx*DATE_BAR_HEIGHT,DATE_BAR_HEIGHT*1024);
     }

   if(m_calc_mode==0)
      CalcHL();
   else
      CalcClose();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::CalcHL(void)
  {
   datetime now[1];
//---    
   if(CopyTime(m_symbol,m_period,1,1,now)<0)
      return;
//---
   datetime t[];
   double   h[],l[];
   long     v[];
//---     - 
   if(m_time_last==now[0])
      return;
//--- ,      
   while(m_time_last!=now[0])
     {
      datetime time_end=m_time_last+PeriodSeconds(m_period)*10000;  
      //---    
      if(CopyHigh(m_symbol,m_period,m_time_last,time_end,h)<0)
         return;
      if(CopyLow(m_symbol,m_period,m_time_last,time_end,l)<0)
         return;
      if(CopyTickVolume(m_symbol,m_period,m_time_last,time_end,v)<0)
         return;
      if(CopyTime(m_symbol,m_period,m_time_last,time_end,t)<0)
         return;
      //---   
      int cnt=ArraySize(h);
      //--- ,       
      if(cnt!=ArraySize(l) ||
         cnt!=ArraySize(t) ||
         cnt!=ArraySize(v))
        {
         return;
        }

      int start_min=ArraySize(m_dates);
      int start_max=start_min+cnt;
      if(ArrayResize(m_dates,start_max,1000)!=start_max)
         return;

      for(int i=start_min;i<start_max;i++)
        {
         m_dates[i].time=t[i-start_min];
         m_dates[i].level=0;
        }
      //---
      //MqlDateTime dl={},dc;
      uint   time=GetTickCount();
      int    dig =SymbolInfoInteger(m_symbol,SYMBOL_DIGITS)<=3?2:4;
      double box =MathPow(10.0,dig);
      int    dtb =start_min,dtl=0;
      //---    
      m_time_last=t[cnt-1]+PeriodSeconds(m_period);
      //---     
      if(m_box!=0)
         box=m_box/box;
      else
         box=DIVR((h[ArrayMaximum(h)]-l[ArrayMinimum(l)])/50,1/box,true)/box;
      //---   / 
      start_min=DIVR(l[ArrayMinimum(l)],box,true)-m_reverse;
      start_max=DIVR(h[ArrayMaximum(h)],box,false)+m_reverse;
      if(start_min<m_global_min) m_global_min=start_min;
      if(start_max>m_global_max) m_global_max=start_max;
      //---  
      if(m_state==0)
        {
         //---  
         start_min=DIVR(l[0],box,true);
         start_max=DIVR(h[0],box,false);
         //---    
         m_rev_min   =start_max-m_reverse;
         m_rev_max   =start_min+m_reverse;
         //---   
         m_cont_min  =start_min-1;
         m_cont_max  =start_max+1;
         //---  
         ArrayResize(m_data,1,cnt);
         ZeroMemory(m_data);
        }
      //---
      dtl=start_min-1;
      //---   
      for(int i=1,n=ArraySize(m_data)-1;i<cnt;i++,dtb++)
        {
         //---     
         int CurMin=DIVR(l[i],box,true);
         int CurMax=DIVR(h[i],box,false);
         m_dates[dtb].level=dtl;
         //---     
         switch(m_state)
           {
            //---   
            case 0:
               //---  
               if(CurMax>=m_rev_max)
                 {
                  m_trend_v   =-1;
                  m_state     ='X';               // 
                  m_cont_max  =CurMax+1;          // 
                  m_rev_min   =CurMax-m_reverse;  // 
                  //---
                  m_data[n].dir =1;
                  m_data[n].beg =start_min-1;
                  m_data[n].end =CurMax   -1;
                  m_data[n].vol+=v[i];
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=start_min-1;
                  //---
                  break;
                 }
               //---  
               if(CurMin<=m_rev_min)
                 {
                  m_trend_v   =1;
                  m_state     ='O';
                  m_cont_min  =CurMin-1;
                  m_rev_max   =CurMin+m_reverse;
                  //---
                  m_data[n].dir =-1;
                  m_data[n].beg =CurMin   -1;
                  m_data[n].end =start_max-1;
                  m_data[n].vol+=v[i];
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=CurMin-1;
                  //---
                  break;
                 }
               //---
               break;

            case 'X':
               //---  
               if(CurMax>=m_cont_max)
                 {
                  m_cont_max    =CurMax+1;
                  m_rev_min     =CurMax-m_reverse;
                  m_data[n].end =CurMax-1;
                  m_data[n].vol+=v[i];
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=CurMax-1;

                  break;
                 }
               //---   ( )
               if(CurMin<=m_rev_min)
                 {
                  m_state      ='O';
                  m_cont_min   =CurMin-1;
                  m_rev_max    =CurMin+m_reverse;
                  m_dates[dtb].level=m_data[n].end;
                  //---  
                  n++;
                  ArrayResize(m_data,n+1);
                  //---  
                  m_data[n].beg =CurMin-1;
                  m_data[n].end =m_data[n-1].end-1;
                  m_data[n].vol =v[i];
                  m_data[n].dir =-1;
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=CurMin-1;
                  break;
                 }
               //---
               break;

            case 'O':
               //---  
               if(CurMin<=m_cont_min)
                 {
                  m_cont_min    =CurMin-1;
                  m_rev_max     =CurMin+m_reverse;
                  m_data[n].beg =CurMin-1;
                  m_data[n].vol+=v[i];
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=CurMin-1;
                  break;
                 }
               //---   ( )
               if(CurMax>=m_rev_max)
                 {
                  m_state      ='X';
                  m_cont_max   =CurMax+1;
                  m_rev_min    =CurMax-m_reverse;
                  m_data[n].dte=dtb;
                  //---  
                  n++;
                  ArrayResize(m_data,n+1);
                  //---  
                  m_data[n].beg =m_data[n-1].beg+1;
                  m_data[n].end =CurMax-1;
                  m_data[n].vol =v[i];
                  m_data[n].dir =1;
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=CurMax-1;
                  break;
                 }
               //---
               break;
           }
        }
      //---   
      ArrayFree(m_trend);
      //---
      if(m_state!=0)
        {
         for(int tv=0,i,j,n=0,beg=1,end=2,ncnt=ArraySize(m_data)-1;end<ncnt;)
           {
            for(i=0;beg+i<end;)
              {
               if(m_trend_v==1)
                 {
                  j=INT_MAX;
                  //---
                  for(int m=beg;m<end;m++)
                     if((m_data[m].beg-1)<j)
                        j=m_data[i=m].beg-1;
                 }
               else
                 {
                  j=INT_MIN;

                  for(int m=beg;m<end;m++)
                     if((m_data[m].end+1)>j)
                        j=m_data[i=m].end+1;
                 }
               //---
               beg=i;
               tv =j;
               i  =0;
               //--
               while((m_data[beg+i].beg)>j || (m_data[beg+i].end)<j)
                 {
                  i++;
                  j+=m_trend_v;
                  if(beg+i==ncnt)
                    {
                     end=ncnt;
                     break;
                    }
                 }
               if(beg+i<end)
                 {
                  beg+=2;
                  i=0;
                 }
              }
            //---
            end=beg+i;
            //---
            if((end-beg)>2)
              {
               n=ArraySize(m_trend);
               ArrayResize(m_trend,n+1);
               m_trend[n].dir =m_trend_v;
               m_trend[n].nbeg=beg;
               m_trend[n].nend=end;
               m_trend[n].vbeg=tv;
               m_trend[n].vend=tv+(end-beg)*m_trend_v;
              }
            //---
            if(end==beg)
               end++;
            //---
            m_trend_v=-m_trend_v;
           }
        }
     }
//---   
//Print(GetTickCount()-time);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::CalcClose(void)
  {
   datetime now[1];
//---    
   if(CopyTime(m_symbol,m_period,1,1,now)<0)
      return;
//---
   datetime t[];
   double   c[];
   long     v[];
//---     - 
   if(m_time_last==now[0])
      return;
//--- ,      
   while(m_time_last!=now[0])
     {
      datetime time_end=m_time_last+PeriodSeconds(m_period)*10000;  
      //---    
      if(CopyClose(m_symbol,m_period,m_time_last,time_end,c)<0)
         return;
      if(CopyTickVolume(m_symbol,m_period,m_time_last,time_end,v)<0)
         return;
      if(CopyTime(m_symbol,m_period,m_time_last,time_end,t)<0)
         return;
      //---   
      int cnt=ArraySize(c);
      //--- ,       
      if(cnt!=ArraySize(t) || cnt!=ArraySize(v))
         return;
      //---
      int start_min=ArraySize(m_dates);
      int start_max=start_min+cnt;
      if(ArrayResize(m_dates,start_max,1000)!=start_max)
         return;
      //---
      for(int i=start_min;i<start_max;i++)
        {
         m_dates[i].time=t[i-start_min];
         m_dates[i].level=0;
        }
      //---
      //MqlDateTime dl={},dc;
      uint   time=GetTickCount();
      double box =MathPow(10.0,m_digits);
      int    dtb =start_min,dtl=0;
      //---    
      m_time_last=t[cnt-1]+PeriodSeconds(m_period);
      //---     
      box=m_box/box;
      //---   / 
      double z=c[ArrayMinimum(c)];
      start_min=DIVR(c[ArrayMinimum(c)],box,false)-m_reverse;
      start_max=DIVR(c[ArrayMaximum(c)],box,false)+m_reverse;
      //---
      if(start_min<m_global_min) m_global_min=start_min;
      if(start_max>m_global_max) m_global_max=start_max;
      //---  
      if(m_state==0)
        {
         //---  
         start_min=start_max=DIVR(c[0],box,false);
         //---    
         m_rev_min   =start_max-m_reverse;
         m_rev_max   =start_min+m_reverse;
         //---   
         m_cont_min  =start_min-1;
         m_cont_max  =start_max+1;
         //---  
         ArrayResize(m_data,1,cnt);
         ZeroMemory(m_data);
        }
      //---
      dtl=start_min-1;
      //---   
      for(int i=1,n=ArraySize(m_data)-1;i<cnt;i++,dtb++)
        {
         //---     
         int Cur=DIVR(c[i],box,false);
         m_dates[dtb].level=dtl;
         //---     
         switch(m_state)
           {
            //---   
            case 0:
               //---  
               if(Cur>=m_rev_max)
                 {
                  m_trend_v   =-1;
                  m_state     ='X';            // 
                  m_cont_max  =Cur+1;          // 
                  m_rev_min   =Cur-m_reverse;  // 
                  //---
                  m_data[n].dir =1;
                  m_data[n].beg =start_min-1;
                  m_data[n].end =Cur      -1;
                  m_data[n].vol+=v[i];
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=start_min-1;
                  //---
                  break;
                 }
               //---  
               if(Cur<=m_rev_min)
                 {
                  m_trend_v   =1;
                  m_state     ='O';
                  m_cont_min  =Cur-1;
                  m_rev_max   =Cur+m_reverse;
                  //---
                  m_data[n].dir =-1;
                  m_data[n].beg =Cur      -1;
                  m_data[n].end =start_max-1;
                  m_data[n].vol+=v[i];
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=Cur-1;
                  //---
                  break;
                 }
               //---
               break;

            case 'X':
               //---  
               if(Cur>=m_cont_max)
                 {
                  m_cont_max    =Cur+1;
                  m_rev_min     =Cur-m_reverse;
                  m_data[n].end =Cur-1;
                  m_data[n].vol+=v[i];
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=Cur-1;

                  break;
                 }
               //---   ( )
               if(Cur<=m_rev_min)
                 {
                  m_state      ='O';
                  m_cont_min   =Cur-1;
                  m_rev_max    =Cur+m_reverse;
                  m_dates[dtb].level=m_data[n].end;
                  //---  
                  n++;
                  ArrayResize(m_data,n+1);
                  //---  
                  m_data[n].beg =Cur-1;
                  m_data[n].end =m_data[n-1].end-1;
                  m_data[n].vol =v[i];
                  m_data[n].dir =-1;
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=Cur-1;
                  break;
                 }
               //---
               break;

            case 'O':
               //---  
               if(Cur<=m_cont_min)
                 {
                  m_cont_min    =Cur-1;
                  m_rev_max     =Cur+m_reverse;
                  m_data[n].beg =Cur-1;
                  m_data[n].vol+=v[i];
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=Cur-1;
                  break;
                 }
               //---   ( )
               if(Cur>=m_rev_max)
                 {
                  m_state      ='X';
                  m_cont_max   =Cur+1;
                  m_rev_min    =Cur-m_reverse;
                  m_data[n].dte=dtb;
                  //---  
                  n++;
                  ArrayResize(m_data,n+1);
                  //---  
                  m_data[n].beg =m_data[n-1].beg+1;
                  m_data[n].end =Cur-1;
                  m_data[n].vol =v[i];
                  m_data[n].dir =1;
                  m_data[n].dtb =dtb;
                  m_data[n].dte =dtb;

                  m_dates[dtb].level=dtl=Cur-1;
                  break;
                 }
               //---
               break;
           }
        }
      //---   
      ArrayFree(m_trend);
      //---
      if(m_state!=0)
        {
         for(int tv=0,i,j,n=0,beg=1,end=2,ncnt=ArraySize(m_data)-1;end<ncnt;)
           {
            for(i=0;beg+i<end;)
              {
               if(m_trend_v==1)
                 {
                  j=INT_MAX;
                  //---
                  for(int m=beg;m<end;m++)
                     if((m_data[m].beg-1)<j)
                        j=m_data[i=m].beg-1;
                 }
               else
                 {
                  j=INT_MIN;

                  for(int m=beg;m<end;m++)
                     if((m_data[m].end+1)>j)
                        j=m_data[i=m].end+1;
                 }
               //---
               beg=i;
               tv =j;
               i  =0;
               //--
               while((m_data[beg+i].beg)>j || (m_data[beg+i].end)<j)
                 {
                  i++;
                  j+=m_trend_v;
                  if(beg+i==ncnt)
                    {
                     end=ncnt;
                     break;
                    }
                 }
               if(beg+i<end)
                 {
                  beg+=2;
                  i=0;
                 }
              }
            //---
            end=beg+i;
            //---
            if((end-beg)>2)
              {
               n=ArraySize(m_trend);
               ArrayResize(m_trend,n+1);
               m_trend[n].dir =m_trend_v;
               m_trend[n].nbeg=beg;
               m_trend[n].nend=end;
               m_trend[n].vbeg=tv;
               m_trend[n].vend=tv+(end-beg)*m_trend_v;
              }
            //---
            if(end==beg)
               end++;
            //---
            m_trend_v=-m_trend_v;
           }
        }
     }
//---   
//Print(GetTickCount()-time);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::KeyDown(int key_code,uint key_state)
  {
   int offset_x=-1;
   int offset_y=-1;

   switch(key_code)
     {
      case 36:
         offset_x=0;
         break;
      case 35:
         offset_x=(ArraySize(m_data)+1)*m_cell_width-m_rect.cx;
         break;
      case 39:
         offset_x=m_offset_x+m_cell_width*30;
         if(offset_x>=(ArraySize(m_data)+1)*m_cell_width-m_rect.cx)
            offset_x=(ArraySize(m_data)+1)*m_cell_width-m_rect.cx-1;
         break;
      case 37:
         offset_x=m_offset_x-m_cell_width*30;
         if(offset_x<0)
            offset_x=0;
         break;
      case 40:  //  
         offset_y=m_offset_y+m_cell_height*5;
         if(offset_y>=(m_global_max-m_global_min)*m_cell_height-m_rect.cy)
            offset_y=(m_global_max-m_global_min)*m_cell_height-m_rect.cy-1;
         break;
      case 38: //  
         offset_y=m_offset_y-m_cell_height*5;
         if(offset_y<0)
            offset_y=0;
         break;
      case 107: // NUM+
         if(m_scale<ArraySize(ExtScaleArray)+1)
           {
            SetScale(++m_scale);
            Redraw();
           }
         break;
      case 109: // NUM-
         if(m_scale>0)
           {
            SetScale(--m_scale);
            Redraw();
           }
         break;
      default:
         //Print(key_code);
         break;
     }
//---  
   if(offset_x>=0 || offset_y>=0)
     {
      bool draw=false;

      if(offset_x>=0 && offset_x!=m_offset_x)
        {
         m_offset_x=offset_x;
         draw=true;
        }
      if(offset_y>=0 && offset_y!=m_offset_y)
        {
         m_offset_y=offset_y;
         draw=true;
        }

      //---
      if(draw)
         Redraw();
      //---
      return;
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::Line(int x1,int y1,int x2,int y2,uint clr)
  {
   if(x1<0 && x2<0) return;
   if(x1>=m_rect.cx && x2>=m_rect.cx) return;
   if(y1<0 && y2<0) return;
   if(y1>=m_rect.cy && y2>=m_rect.cy) return;

   int dx=x2-x1;
   int dy=y2-y1;
   double k;
//---
   if(fabs(dx)>fabs(dy))
     {
      k=dy/(double(dx)+0.0000001);
      if(x2<x1) { dx=x2; x2=x1; x1=dx; dy=y2; y2=y1; y1=dy; }
      if(x1>=m_rect.cx || x2<0) return;
      if(x1<0) { y1+=int(k*-x1); x1=0; }
      if(x2>=m_rect.cx) x2=m_rect.cx-1;

      for(dx=x1;dx<=x2;dx++)
        {
         dy=y1+int((dx-x1)*k);
         //---
         if(dy>0  &&  dy<m_rect.cy)
            m_surface[dy*m_rect.cx+dx]=clr;
        }
     }
   else
     {
      k=dx/(double(dy)+0.0000001);
      if(y2<y1) { dy=y2; y2=y1; y1=dy; dx=x1; x1=x2; x2=dx; }
      if(y1>=m_rect.cy || y2<0) return;
      if(y1<0) { x1+=int(k*-y1); y1=0; }
      if(y2>=m_rect.cy) y2=m_rect.cy-1;

      for(dy=y1;dy<=y2;dy++)
        {
         dx=x1+int((dy-y1)*k);
         //---
         if(dx>0  &&  dx<m_rect.cx)
            m_surface[dy*m_rect.cx+dx]=clr;
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::DrawTrendLines(int v_shift)
  {
   int height=(m_global_max-m_global_min)*m_cell_height;
   int count =ArraySize(m_trend);
   int yo    =height-v_shift;
   int ch2   =m_cell_height*2;
//---   
   for(int i=0;i<count;i++)
     {
      int x1=(m_trend[i].nbeg*m_cell_width-m_offset_x)+m_cell_width/2;
      int x2=(m_trend[i].nend*m_cell_width-m_offset_x)-m_cell_width/2;
      //---     
      if(x2<0 || x1>=m_rect.cx)
         continue;
      //---
      int y1=yo-(m_trend[i].vbeg-m_global_min)*m_cell_height-m_cell_height/2;
      int y2=yo-(m_trend[i].vend-m_global_min)*m_cell_height+m_cell_height/2;
      if(m_trend[i].dir<0) y2-=ch2;
      //---   
      Line(x1,y1,x2,y2,m_trend[i].dir<0?0xFF0000:0x008000);
      //---  
      if(m_trend[i].dir<0)
         Line(x1,y1-1,x2,y2-1,0xFF0000);
      else
         Line(x1,y1+1,x2,y2+1,0x008000);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::DrawPrices(int v_shift)
  {
   if(ArraySize(m_data)==0)
      return;
  
   double box   =m_box/MathPow(10.0,m_digits);
   double price =m_global_max*box;
   int    count =m_global_max-m_global_min;
   int    height=count*m_cell_height-v_shift+m_cell_height/2;
//---
   if(m_scale<2)
      TextSetFont("Arial",ExtScaleArray[m_scale]);
   else
      TextSetFont("Arial",FONT_SIZE);
//--- 
   while((height-count*m_cell_height-m_cell_height/2)>0)
     {
      price+=box;
      count++;
     }
//---
   for(int i=count;;price-=box)
     {
      TextOut(DoubleToString(price,m_digits),2,height-i*m_cell_height-m_cell_height/2,TA_LEFT|TA_VCENTER,m_graph_p,
              m_price_width,m_rect.cy,PRICE_COLOR,COLOR_FORMAT_XRGB_NOALPHA);
      //---
      i--;
      if((height-i*m_cell_height-m_cell_height/2)>m_rect.cy)
         break;
     } 
//---
   if(m_tick.bid>0.0)
     {
      double f=m_box/MathPow(10.0,m_digits);
      int    b=DIVR(m_tick.bid,f,false);
      //---
      f=(m_tick.bid-b*f)*(m_cell_height/f);
      b=(m_global_max-b)*m_cell_height-v_shift-int(f+0.5);
      if(b>=0 && b<m_rect.cy)
        {
         int y=b-5,n=b+5;
         if(y<0) y=0;
         if(n>=m_rect.cy) n=m_rect.cy-1;        
         for(;y<=n;y++)
            ArrayFill(m_graph_p,y*m_price_width,m_price_width,0xFFE0A0A0);
         //---
         TextSetFont("Arial",FONT_SIZE);
         TextOut(DoubleToString(m_tick.bid,m_digits),m_price_width/2,b+1,TA_CENTER|TA_VCENTER,m_graph_p,m_price_width,m_rect.cy,0,COLOR_FORMAT_XRGB_NOALPHA);
        }
      
     }     
  }
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
void CXOGraph::DrawBars(int v_shift)
  {
//---
   if(m_tick.bid>0.0)
     {
      double f=m_box/MathPow(10.0,m_digits);
      int    b=DIVR(m_tick.bid,f,false);
      //---
      f=(m_tick.bid-b*f)*(m_cell_height/f);
      b=(m_global_max-b)*m_cell_height-v_shift-int(f+0.5);
      if(b>=0 && b<m_rect.cy)
        {
         for(int i=b-1,j=5;i>0 && j>0;i--,j--)
            ArrayFill(m_surface,i*m_rect.cx+m_rect.cx-int(j*1.7),int(j*1.7),0xFFE0A0A0);
         for(int i=b+1,j=5;i<m_rect.cy && j>0;i++,j--)
            ArrayFill(m_surface,i*m_rect.cx+m_rect.cx-int(j*1.7),int(j*1.7),0xFFE0A0A0);
        
         ArrayFill(m_surface,b*m_rect.cx,m_rect.cx,0xFFE0A0A0);         
        }
      
     }
//---
   int yo   =(m_global_max-m_global_min)*m_cell_height-v_shift-m_cell_height/2;
   int xo   =m_cell_width/2-m_offset_x;
   int count=ArraySize(m_data);
//---   
   if(count>((m_rect.cx+m_offset_x)/m_cell_width))
      count=(m_rect.cx+m_offset_x)/m_cell_width;
//---  
   TextSetFont("Arial",m_font_size,FW_BOLD|FONT_ITALIC);
//---   
   for(int i=m_offset_x/m_cell_width;i<count;i++)
     {
      int      dir=m_data[i].dir;
      string   sig=dir<0?"O":"X";
      uint     clr=dir<0?0xAF0000:0x008000;
      int      beg=m_data[i].beg-m_global_min;
      int      end=m_data[i].end-m_global_min;
      int      x  =xo+i*m_cell_width;
      //---
      for(int y=beg;y<=end;y++)
         TextOut(sig,x,yo-y*m_cell_height,TA_CENTER|TA_VCENTER,m_surface,m_rect.cx,m_rect.cy,clr,COLOR_FORMAT_XRGB_NOALPHA);
     }
  }
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
void CXOGraph::DrawDates(int v_shift)
  {
//---    
   string str;
   MqlDateTime dl={},dc;
   int count     =ArraySize(m_data);
   int height    =(m_global_max-m_global_min)*m_cell_height;
   int xo        =m_cell_width/2-m_offset_x;
//---   
   if(count>((m_rect.cx+m_offset_x)/m_cell_width))
      count=(m_rect.cx+m_offset_x)/m_cell_width;
//---
   if(m_scale<3)
      TextSetFont("Arial",ExtScaleArray[m_scale],FW_NORMAL,2700);
   else
      TextSetFont("Arial",FONT_SIZE,FW_NORMAL,2700);
//---   
   for(int i=m_offset_x/m_cell_width;i<count;i++)
     {
      //---    
      TimeToStruct(m_dates[m_data[i].dte].time,dc);
      //---
      int x=xo+i*m_cell_width;
      //---  
      if(dl.mon!=dc.mon)
        {
         dl=dc;
         //---
         str=StringFormat("%04d.%02d",dc.year,dc.mon);
         TextOut(str,x,4,TA_LEFT|TA_VCENTER,m_graph_d,m_rect.cx,DATE_BAR_HEIGHT,(dc.mon==1)?0xB00000:0x808000,COLOR_FORMAT_XRGB_NOALPHA);
         //---   
         if(x>=0 && x<m_rect.cx)
            for(int j=0;j<m_rect.cy;j++)
               if(((j+v_shift)&2)==2)
                  m_surface[j*m_rect.cx+x]=0xA0A0A0;
        }
      else
        {
         if(dl.day!=dc.day)
           {
            str=StringFormat("%02d",dc.day);
            TextOut(str,x,4,TA_LEFT|TA_VCENTER,m_graph_d,m_rect.cx,DATE_BAR_HEIGHT,PRICE_COLOR,COLOR_FORMAT_XRGB_NOALPHA);
            dl=dc;
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CXOGraph::SetScale(int x)
  {
   if(x<0) x=0;
   if(x>=ArraySize(ExtScaleArray)) x=ArraySize(ExtScaleArray)-1;
//---
   m_font_size=ExtScaleArray[x];
   m_scale    =x;
//---
   TextSetFont("Arial",m_font_size);
//---
   if(!TextGetSize("X",m_cell_width,m_cell_height))
      m_cell_width=10;
//---
   m_cell_width =m_cell_width*3/2;
   m_cell_height=m_cell_width;

   //Print(m_scale);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CXOGraph::CalcVShift(void)
  {
   int count=ArraySize(m_data);
   int yo   =(m_global_max-m_global_min);
   int y_min=INT_MAX;
   int y_max=INT_MIN;
//---   
   if(count>((m_rect.cx+m_offset_x)/m_cell_width))
      count=(m_rect.cx+m_offset_x)/m_cell_width;
//---   - 
   if(count==0)
      return(0);
//---
   int i      =m_offset_x/m_cell_width;
   int x_count=m_rect.cx/m_cell_width;
   if(i+x_count>count)
     {
      i=count-x_count;
      if(i<0) i=0;
      m_offset_x=i*m_cell_width;
     }
//---   
   for(;i<count;i++)
     {
      int beg=m_data[i].beg;
      int end=m_data[i].end+1;
      if(y_min>beg) y_min=beg;
      if(y_max<end) y_max=end;
     }
//---   
   int height=(y_max-y_min)*m_cell_height;
//---    
   if(height<m_rect.cy)
      return((yo-(y_max-m_global_min))*m_cell_height-(m_rect.cy-height)/2);
//---    2    2 
   y_min-=2;
   y_max+=2;
//---
   int val=((yo-(y_max-m_global_min)))*m_cell_height+m_offset_y;
   int max=((yo-(y_min-m_global_min)))*m_cell_height;
//--- 
   if(val+m_rect.cy>max)
      val=max-m_rect.cy;
//---
   return(val);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CXOGraph::OnNotify(int code,CFrame *sender)
  {
   if(sender.Id()==1000)
     {
      UpdateParameters(m_tool_bar.GetBoxValue(),m_tool_bar.GetRevValue(),m_tool_bar.GetDateValue(),m_tool_bar.GetModeValue());
      NewTick();
     }
//---
   return(true);
  }
//+------------------------------------------------------------------+
