
#include "deftypes.mqh"

enum EnCFrame
  {
   FRAME_LIST_PARENT    =0,
   FRAME_LIST_CHILD     =1,
   FRAME_LIST_PREV      =2,
   FRAME_LIST_NEXT      =3
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CFrame
  {
   CFrame           *m_tree[4];

protected:
   uint              m_id;
   SRect             m_rect;
  
public:
                     CFrame();
                    ~CFrame();
   //---
   void              Destroy();
   uint              Id() const { return(m_id); }
   bool              NotifyParent(int code);
   //---
   void              AddChild(CFrame *child);
   void              DelChild(CFrame *child);
   //---
   virtual bool      OnMouseMove(int x,int y,uint state) { return(false); }
   virtual void      OnPaint(uint &buffer[],int width,int height);
   virtual void      OnCreate() { }
   virtual bool      OnNotify(int code,CFrame *sender) { return(false); }
   //---
   virtual bool      EventMouse(int x,int y,uint state);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFrame::CFrame(void)
  {
   m_tree[0]=NULL;
   m_tree[1]=NULL;
   m_tree[2]=NULL;
   m_tree[3]=NULL;

   m_rect.Clear();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFrame::~CFrame(void)
  {
   Destroy();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CFrame::AddChild(CFrame *child)
  {
//---  
   if(CheckPointer(child))
     {
      //---    -    
      if(CheckPointer(child.m_tree[FRAME_LIST_PARENT]))
         child.m_tree[FRAME_LIST_PARENT].DelChild(child);
      //---
      child.m_tree[FRAME_LIST_NEXT]=m_tree[FRAME_LIST_CHILD];
      child.m_tree[FRAME_LIST_PREV]=NULL;
      if(CheckPointer(m_tree[FRAME_LIST_CHILD]))
         m_tree[FRAME_LIST_CHILD].m_tree[FRAME_LIST_PREV]=child;
      m_tree[FRAME_LIST_CHILD]=child;
      child.m_tree[FRAME_LIST_PARENT]=GetPointer(this);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CFrame::DelChild(CFrame *child)
  {
   CFrame *next;
//---
   if(CheckPointer(child))
     {
      next=m_tree[FRAME_LIST_CHILD];
      //---
      for(;CheckPointer(next);next=next.m_tree[FRAME_LIST_NEXT])
        {
         if(next==child)
           {
            //---
            if(CheckPointer(next.m_tree[FRAME_LIST_PREV]))
              {
               next.m_tree[FRAME_LIST_PREV].m_tree[FRAME_LIST_NEXT]=next.m_tree[FRAME_LIST_NEXT];
               //---
               if(next==m_tree[FRAME_LIST_CHILD])
                  m_tree[FRAME_LIST_CHILD]=next.m_tree[FRAME_LIST_PREV];
              }
            //---
            if(CheckPointer(next.m_tree[FRAME_LIST_NEXT]))
              {
               next.m_tree[FRAME_LIST_NEXT].m_tree[FRAME_LIST_PREV]=next.m_tree[FRAME_LIST_PREV];
               if(next==m_tree[FRAME_LIST_CHILD])
                  m_tree[FRAME_LIST_CHILD]=next.m_tree[FRAME_LIST_NEXT];
              }
            //---
            if(next==m_tree[FRAME_LIST_CHILD])
               m_tree[FRAME_LIST_CHILD]=NULL;
            //---
            return;
           }
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CFrame::EventMouse(int x,int y,uint state)
  {
//---     
   CFrame *child=m_tree[FRAME_LIST_CHILD];
   for(;CheckPointer(child);child=child.m_tree[FRAME_LIST_NEXT])
      if(child.EventMouse(x,y,state))
         return(true);
//---   
   if(!m_rect.PtInRect(x,y))
      return(false);
//---
   return(OnMouseMove(x,y,state));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CFrame::OnPaint(uint &buffer[],int width,int height)
  {
   
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CFrame::Destroy(void)
  {
   if(CheckPointer(m_tree[FRAME_LIST_PARENT]))
      m_tree[FRAME_LIST_PARENT].DelChild(GetPointer(this));  
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CFrame::NotifyParent(int code)
  {
   if(CheckPointer(m_tree[FRAME_LIST_PARENT]))
      return(m_tree[FRAME_LIST_PARENT].OnNotify(code,GetPointer(this)));
//---
   return(false);
  }
//+------------------------------------------------------------------+
